const CLT_ConsoleService = Components.classes['@mozilla.org/consoleservice;1'].getService(Components.interfaces.nsIConsoleService);
const CLT_PrefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
const CLT_Branch = CLT_PrefService.getBranch("colt.");

// Preference names (as they appear in the "colt" preferences branch)
var CLT_PrefName_ShowCopyText		= "showcopytext";
var CLT_PrefName_ShowCopyBoth		= "showcopyboth";
var CLT_PrefName_ShowCopyPage		= "showcopypage";
var CLT_PrefName_CustomFormatCount 	= "custom.count";
var CLT_PrefName_Version			= "version";

// Option variables
var CLT_ShowCopyText = false;
var CLT_ShowCopyBoth = false;
var CLT_ShowCopyPage = false;

// Miscellaneous variables
var CLT_Once = false;
var CLT_Version = "2.4"; // Must be of the form "X.Y" or "X.YZ" (only 1 decimal)

function CLT_Log(aMessage)
{
    CLT_ConsoleService.logStringMessage('CoLT: ' + aMessage);
}

function CLT_CopyBoth(formatIndex, type)
{
	var url;
	var text;
	var titleAttr;
	var pageTitle;
	var pageURL;

	if(type == "page")
	{
		url = content.document.location;
		text = content.document.title;
		titleAttr = "";
		pageTitle = "";
		pageURL = "";
	}
	else
	{
		url = gContextMenu.linkURL;
		text = gContextMenu.linkText();
		titleAttr = document.popupNode.title;
		pageTitle = content.document.title;
		pageURL = content.document.location;
	}
	
	var format = CLT_Branch.getCharPref("custom." + formatIndex + ".format");
	var myString = CLT_FormatString(format, text, url, titleAttr, pageTitle, pageURL);

	var result = CLT_PlaceOnClipboard(myString);
	if(!result)
		alert("ERROR: The link text and location were unable to be placed on the clipboard.");
}

function CLT_CopyBothAsRichText(type)
{
	var richText = "<a href=\"";

	if(type == "page")
		richText += content.document.location + "\">" + content.document.title + "</a>";
	else
		richText += gContextMenu.linkURL + "\">" + gContextMenu.linkText() + "</a>";
	
	var xfer = Components.classes["@mozilla.org/widget/transferable;1"].createInstance(Components.interfaces.nsITransferable);
	xfer.addDataFlavor("text/html");

	var htmlString = Components.classes["@mozilla.org/supports-string;1"].createInstance(Components.interfaces.nsISupportsString);
	htmlString.data = richText;
	xfer.setTransferData("text/html", htmlString, richText.length * 2);

	var clipboard = Components.classes["@mozilla.org/widget/clipboard;1"].getService(Components.interfaces.nsIClipboard);
	clipboard.setData(xfer, null, Components.interfaces.nsIClipboard.kGlobalClipboard);
}

function CLT_CopyLinkText()
{
	CLT_Log("Element: " + document.popupNode);
	
	var linkText = gContextMenu.linkText();
	var result = CLT_PlaceOnClipboard(linkText);
	if(!result)
		alert("ERROR: The link text was unable to be placed on the clipboard.");
}

function CLT_FormatString(string, text, url, titleAttr, pageTitle, pageURL)
{
	string = string.replace(/%[Tt]/g, text);
	string = string.replace(/%[Uu]/g, url);
	string = string.replace(/%[Nn]/g, CLT_GetNewLine());
	string = string.replace(/%[Ii]/g, titleAttr);
	string = string.replace(/%[Pp]/g, pageTitle);
	string = string.replace(/%[Rr]/g, pageURL);
	return string;
}

function CLT_GetNewLine()
{
	var platform = navigator.platform.toLowerCase();

	if(platform.indexOf('win') != -1) // Windows
		return "\r\n";

	else if(platform.indexOf('mac') != -1) // Mac
		return "\r";

	else // *nix
		return "\n";
}

function CLT_Init()
{
	if(CLT_Once == false)
	{
		CLT_Once = true;

		CLT_Upgrade();
		CLT_LoadPrefs();
		CLT_UpdateContextMenu();

		var menu = document.getElementById("contentAreaContextMenu");
		menu.addEventListener('popupshowing', CLT_UpdateContextMenu, false);
	}
}

function CLT_IsPreferenceSet(pref)
{
	if(pref)
		return CLT_Branch.prefHasUserValue(pref);

	return false;
}

function CLT_LoadPrefs()
{
	var b = CLT_Branch;
	var initDefaults = false;

	CLT_ShowCopyText = b.getBoolPref(CLT_PrefName_ShowCopyText);
	CLT_ShowCopyBoth = b.getBoolPref(CLT_PrefName_ShowCopyBoth);
	CLT_ShowCopyPage = b.getBoolPref(CLT_PrefName_ShowCopyPage);
	CLT_CustomFormatCount = b.getIntPref(CLT_PrefName_CustomFormatCount);
}

function CLT_MigrateTo23()
{
	var stringBundle = document.getElementById("CLT-String-Bundle");

	// Create all the new preferences, migrating old settings as necessary
	CLT_Branch.setIntPref(CLT_PrefName_CustomFormatCount, 6);

	CLT_Branch.setCharPref("custom.1.label", stringBundle.getString("CLT_DefaultLabelHTMLLink"));
	CLT_Branch.setCharPref("custom.1.format", "<a href=\"%U\">%T</a>");
	CLT_Branch.setBoolPref("custom.1.separator", false);
	
	CLT_Branch.setCharPref("custom.2.label", stringBundle.getString("CLT_DefaultLabelPlainText"));
	CLT_Branch.setCharPref("custom.2.format", "%T - %U");
	CLT_Branch.setBoolPref("custom.2.separator", false);

	CLT_Branch.setBoolPref("custom.3.separator", true);

	if(CLT_IsPreferenceSet("customlabel1") && CLT_IsPreferenceSet("customformat1"))
	{
		CLT_Branch.setCharPref("custom.4.label", CLT_Branch.getCharPref("customlabel1"));
		CLT_Branch.setCharPref("custom.4.format", CLT_Branch.getCharPref("customformat1"));
		CLT_Branch.setBoolPref("custom.4.separator", false);

		if(CLT_Branch.prefHasUserValue("customlabel1"))
			CLT_Branch.clearUserPref("customlabel1");

		if(CLT_Branch.prefHasUserValue("customformat1"))
			CLT_Branch.clearUserPref("customformat1");
	}
	else
	{
		// If we can't migrate, set a default custom format
		CLT_Branch.setCharPref("custom.4.label", "BB Code");
		CLT_Branch.setCharPref("custom.4.format", "[url=%U]%T[/url]");
		CLT_Branch.setBoolPref("custom.4.separator", false);
	}

	if(CLT_IsPreferenceSet("customformat2") && CLT_IsPreferenceSet("customlabel2"))
	{
		CLT_Branch.setCharPref("custom.5.label", CLT_Branch.getCharPref("customlabel2"));
		CLT_Branch.setCharPref("custom.5.format", CLT_Branch.getCharPref("customformat2"));
		CLT_Branch.setBoolPref("custom.5.separator", false);

		if(CLT_Branch.prefHasUserValue("customlabel2"))
			CLT_Branch.clearUserPref("customlabel2");

		if(CLT_Branch.prefHasUserValue("customformat2"))
			CLT_Branch.clearUserPref("customformat2");
	}
	else
	{
		// If we can't migrate, set a default custom format
		CLT_Branch.setCharPref("custom.5.label", "FuseTalk");
		CLT_Branch.setCharPref("custom.5.format", "[L=%T]%U[/L]");
		CLT_Branch.setBoolPref("custom.5.separator", false);
	}

	if(CLT_IsPreferenceSet("customformat3") && CLT_IsPreferenceSet("customlabel3"))
	{
		CLT_Branch.setCharPref("custom.6.label", CLT_Branch.getCharPref("customlabel3"));
		CLT_Branch.setCharPref("custom.6.format", CLT_Branch.getCharPref("customformat3"));
		CLT_Branch.setBoolPref("custom.6.separator", false);

		if(CLT_Branch.prefHasUserValue("customlabel3"))
			CLT_Branch.clearUserPref("customlabel3");

		if(CLT_Branch.prefHasUserValue("customformat3"))
			CLT_Branch.clearUserPref("customformat3");
	}
	else
	{
		// If we can't migrate, set a default custom format
		CLT_Branch.setCharPref("custom.6.label", "Wikipedia");
		CLT_Branch.setCharPref("custom.6.format", "[%U %T]");
		CLT_Branch.setBoolPref("custom.6.separator", false);
	}

	// Delete all other old preferences that we no longer use
	if(CLT_IsPreferenceSet("submenu.custom1"))
		CLT_Branch.clearUserPref("submenu.custom1");

	if(CLT_IsPreferenceSet("submenu.custom2") && CLT_Branch.prefHasUserValue("submenu.custom2"))
		CLT_Branch.clearUserPref("submenu.custom2");

	if(CLT_IsPreferenceSet("submenu.custom3") && CLT_Branch.prefHasUserValue("submenu.custom3"))
		CLT_Branch.clearUserPref("submenu.custom3");

	if(CLT_IsPreferenceSet("submenu.html") && CLT_Branch.prefHasUserValue("submenu.html"))
		CLT_Branch.clearUserPref("submenu.html");

	if(CLT_IsPreferenceSet("submenu.text") && CLT_Branch.prefHasUserValue("submenu.text"))
		CLT_Branch.clearUserPref("submenu.text");
}

function CLT_MigrateTo24()
{
	var count = CLT_Branch.getIntPref(CLT_PrefName_CustomFormatCount);
	for (var i=1; i <= count; i++)
	{
		var prefName = "custom." + i + ".richtext";
		CLT_Branch.setBoolPref(prefName, false);
	}
}

function CLT_OptionsHaveUpdated()
{
	CLT_LoadPrefs();
	CLT_UpdateContextMenu();
}

function CLT_ParseVersion(version)
{
	if(version)
	{
		var splitVersion = version.split(".");
		var parsedVersion = splitVersion[0] + ".";

		for(var i=1; i<splitVersion.length; i++)
		{
			parsedVersion += splitVersion[i];
		}

		return parseFloat(parsedVersion);
	}
	else
		return 0;
}

function CLT_PlaceOnClipboard(string)
{
	var clipboard = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);
	clipboard.copyString(string);
	return true;
}

function CLT_PurgeContextSubMenu(nodeID)
{
	// Remove all the menu items from the sub menu (since we will recreate them)
	var popupmenu = document.getElementById(nodeID);
	while(popupmenu.firstChild)
		popupmenu.removeChild(popupmenu.firstChild);
}

function CLT_UpdateContextMenu()
{
	// Only show the menu items if we're on a link and it's not a mailto link
	var hiddenFlag = true;
	if(gContextMenu && (gContextMenu.onLink && !gContextMenu.onMailtoLink))
		hiddenFlag = false;

	var copyText = document.getElementById("CLT-Context-CopyLinkText");
	var copyBothItem = document.getElementById("CLT-Context-CopyBoth");
	var copyBothMenu = document.getElementById("CLT-Context-CopyBothMenu");
	var copyPageItem = document.getElementById("CLT-Context-CopyPage");
	var copyPageMenu = document.getElementById("CLT-Context-CopyPageMenu");

	copyText.hidden = (CLT_ShowCopyText) ? hiddenFlag : true;
	copyBothItem.hidden = (CLT_ShowCopyBoth && (CLT_CustomFormatCount == 1)) ? hiddenFlag : true;
	copyBothMenu.hidden = (CLT_ShowCopyBoth && (CLT_CustomFormatCount > 1)) ? hiddenFlag : true;

	// This time we default to false (we want to show the items more often than we want to hide them)
	hiddenFlag = false;
	if(gContextMenu && (gContextMenu.onLink || gContextMenu.onTextInput))
		hiddenFlag = true;
	
	copyPageItem.hidden = (CLT_ShowCopyPage && (CLT_CustomFormatCount == 1)) ? hiddenFlag : true;
	copyPageMenu.hidden = (CLT_ShowCopyPage && (CLT_CustomFormatCount > 1)) ? hiddenFlag : true;
}

function CLT_UpdateContextSubMenu(nodeID, type)
{
	CLT_PurgeContextSubMenu(nodeID);

	var popupmenu = document.getElementById(nodeID);
	
    for(var i=1; i <= CLT_CustomFormatCount; i++)
	{
		var separatorPref = "custom." + i + ".separator";

		var isSeparator = false;
        if(CLT_IsPreferenceSet(separatorPref))
			isSeparator = CLT_Branch.getBoolPref(separatorPref);
		
		if(isSeparator)
		{
			var menuseparator = document.createElement("menuseparator");
			popupmenu.appendChild(menuseparator);
		}
		else
		{
			var isRichText = false;
			var richTextPref = "custom." + i + ".richtext";
			if(CLT_IsPreferenceSet(richTextPref))
				isRichText = CLT_Branch.getBoolPref(richTextPref);

			var labelPref = "custom." + i + ".label";
			var label = "";
			if(CLT_IsPreferenceSet(labelPref))
				label = CLT_Branch.getCharPref(labelPref);

			// Skip any weird occurances that don't have a label (shouldn't happen, but you never know)
			if(label && label != "")
			{
				var menuitem = document.createElement("menuitem");
				menuitem.setAttribute("label", label);

				if(isRichText)
					menuitem.setAttribute("oncommand", "CLT_CopyBothAsRichText('" + type + "');");
				else
					menuitem.setAttribute("oncommand", "CLT_CopyBoth('" + i + "', '" + type + "');");

				popupmenu.appendChild(menuitem);
			}
		}
	}
}

function CLT_Upgrade()
{
	var previousVersion = 0;
	
	if(CLT_IsPreferenceSet(CLT_PrefName_Version))
		previousVersion = CLT_ParseVersion(CLT_Branch.getCharPref(CLT_PrefName_Version));

	var currentVersion = CLT_ParseVersion(CLT_Version);

	if(previousVersion != currentVersion)
	{
		if(previousVersion < CLT_ParseVersion("2.3"))
			CLT_MigrateTo23();

		if(previousVersion < CLT_ParseVersion("2.4"))
			CLT_MigrateTo24();
		
		CLT_Branch.setCharPref(CLT_PrefName_Version, CLT_Version);
	}
}
